/*
    Project: TCGINA
    Version: V1.25
    File   : mediumdlg.c

    Copyright (c) 2005-2008 Author of TCGINA DLL. All rights reserved.
    Public Key Fingerprint: 294B A769 4A0A CC05 DAE6 00DD FF47 8C72 4097 67CE

    Covered by TrueCrypt License 2.3 the full text of which is contained
    in the file License.txt included in TCGINA binary and source code
    distribution archives.

    The copyright notice of this source code file and the public key
    fingerprint must be retained in full.
*/

#include <windows.h>
#include <tchar.h>
#include "misc.h"
#include "mediumdlg.h"

#define REFRESH_RATE 100

INT_PTR CALLBACK MediumDialogProc(HWND hwndDlg, UINT uMsg,
  WPARAM wParam, LPARAM lParam)
{
    PMEDIUM_DIALOG_DATA pmdd;

    switch (uMsg) {
    case WM_INITDIALOG:
        pmdd = (PMEDIUM_DIALOG_DATA) lParam;
        pmdd->dwStartTime = GetTickCount();
        if (pmdd->pszCaption) {
            SetWindowText(hwndDlg, pmdd->pszCaption);
        }
        if (pmdd->pszText) {
            HANDLE hwnd = GetDlgItem(hwndDlg, IDC_TEXT);
            SetWindowText(hwnd, pmdd->pszText);
        }
        if (pmdd->pfMediumAvailable == NULL) {
            EndDialog(hwndDlg, IDCANCEL);
            return TRUE;
        }
        SetWindowUserData(hwndDlg, pmdd);
        SetTimer(hwndDlg, 0, REFRESH_RATE, 0);
        return TRUE;

    case WM_TIMER: {
        pmdd = (PMEDIUM_DIALOG_DATA) GetWindowUserData(hwndDlg);

        if (pmdd->pfMediumAvailable(pmdd->pContext)) {
            PostMessage(hwndDlg, WM_COMMAND, IDOK, 0);
        } else if (pmdd->dwTimeout) {
            DWORD dwTime = GetTickCount();
            DWORD dwExpirationTime = pmdd->dwTimeout;
            if (dwTime > pmdd->dwStartTime) {
                dwExpirationTime += pmdd->dwStartTime;
            }
            if (dwTime > dwExpirationTime) {
                PostMessage(hwndDlg, WM_COMMAND, IDCANCEL, 0);
                pmdd->dwTimeout = 0;
            }
        }
        return TRUE; }

    case WM_COMMAND:
        switch (LOWORD(wParam)) {
        case IDOK:
        case IDCANCEL:
            KillTimer(hwndDlg, 0);
            EndDialog(hwndDlg, LOWORD(wParam));
            return TRUE;
        }
        break;
    }

    return FALSE;
}
